@@ -13,15 +13,21 @@
 ##  A copy of the GNU General Public License is available at
 ##  http://www.r-project.org/Licenses/
 
-#' @include controller.R
+##' @include controller.R
 roxygen()
 
 ## Layouts for specifying how items get put
 
-## Container objects are for layouts.
-#' Base Trait for Container objects.
-#'
-#' Basic container is a glayout object for tabular layout
+##' Base Trait for Container objects. Containers are used to make views.
+##'
+##' Basic container is a glayout object for tabular layout
+##' There are various types of layouts. The most basic, and default view, is simply
+##' \code{aContainer(...names of items...)} which simply uses a table to display the item's label
+##' and editor. Other containers can be used to adjust this.
+##'
+##' Containers have a few methods, notably the \code{is_visible} and
+##' \code{is_enabled} methods which can be used to hide or set
+##' sensitive to  user input the container's components.
 Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
                               .doc_container=paste(
                                 desc("property to store base gWidgets container")
@@ -29,7 +35,7 @@ Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
                               container=NULL,  # stores base container
                               children=list(),     # stores children
                               type="glayout",
-                              attr=list(horizontal=FALSE),
+                              attr=list(),
                               .doc_context=paste(
                                 desc("Context for container to look up values, such as whether to draw a label",
                                      "Usually inherited from calling object")
@@ -56,7 +62,13 @@ Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
 #                                  .$container <- cont
                                   return(container) # for aContext
                                 }
+
                                 attr$container <- container
+
+                                if(.$type == "glayout")
+                                  attr$spacing=0
+                                
+
                                 if(inherits(container,"gLayout") ||
                                    (inherits(container,"guiContainer") && grepl("^gLayout", class(container@widget)))
                                    ) {
@@ -65,7 +77,12 @@ Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
                                   no_cols <- get_with_default(tag(container, "no_cols"), 1)
                                   row_no <- tag(container, "row_no")
 
+                                  theSize <- attr$size; attr$size <- NULL
                                   container[row_no, 2*(col_no-1) + 1:2] <- (new_cont <- do.call(.$type, attr))
+                                  visible(new_cont) <- TRUE
+                                  
+                                  if(!is.null(theSize))
+                                    size(new_cont) <- theSize
                                   tag(container, "row_no") <- row_no + (col_no == no_cols)
                                   tag(container, "col_no") <- (col_no %% no_cols) + 1
                                 } else {
@@ -75,7 +92,11 @@ Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
                                      ) {
                                     attr$label <- get_with_default(.$label, "")
                                   }
+                                  theSize <- attr$size; attr$size <- NULL
                                   new_cont <- do.call(.$type, attr)
+                                  visible(new_cont) <- TRUE
+                                  if(!is.null(theSize))
+                                    size(new_cont) <- theSize
                                   ## catch gWidgetsWwW and other
                                   if(inherits(new_cont,"gLayout") ||
                                      (inherits(new_cont,"guiContainer") &&grepl("^gLayout", class(new_cont@widget)))
@@ -89,7 +110,7 @@ Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
                                 new_cont
                               },
                              .doc_init_ui=paste(
-                               desc("If non <code>NULL</cod> called on intial drawing of container")
+                               desc("If non <code>NULL</code> called on intial drawing of container")
                                ),
                              init_ui=function(.) {},
                               ## context an ItemGroup instance to look up strings in.
@@ -107,10 +128,9 @@ Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
                                   else
                                     context <- .
                                 }
-
-                                cont <- .$make_container(container, attr)
+                                cont <- .$make_container(container)
+                                
                                 sapply(.$children, function(i) {
-
                                   if(is.null(i))
                                     return()
 
@@ -127,11 +147,13 @@ Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
                                   }
                                   ## Now dispatch one of several ways.
                                   ## either an Item, ItemGroup or Container
+
+                                  
                                   if(is.proto(i) && i$is("Item")) {
                                     i$make_ui(cont)
                                     i$init_ui()
                                   } else if(is.proto(i) && i$is("ItemGroup")) {
-                                    i$make_ui(cont)
+                                    i$make_ui(cont) ## XXX, gui_layout=i$make_default_gui_layout())
                                   } else if(is.proto(i) && i$is("Container")) {
                                     ## if non-trivial context in container, pass in
                                     if(i$has_slot("context") && !is.null(i$context)) {
@@ -229,19 +251,30 @@ Container <- BaseTrait$proto(class=c("Container", BaseTrait$class),
 ## Various container constructors
 
 ## this is technical to give some children a context
-#' A container to give a different context than the default for a set of items
-#'
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' A container to give a different context than the default for a set of items
+##'
+##' The basic container uses the calling model (a dialog or
+##' item group) as its context. This allows the context to be
+##' overridden, which might be desirable if the items are in more than
+##' one dialog.
+##'
+##' @param context ItemGroup or item to get context from. Typically just NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' 
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @seealso \code{\link{Container}}
+##' @export
+##' 
 aContext <- function(..., 
-                     context, enabled_when, visible_when) {
+                     context, attr=list(),
+                     enabled_when, visible_when) {
   obj <- Container$proto(children=list(...),
-                  type="",              # empty so no parent made
-                  context=context)
+                         attr=attr,
+                         type="",              # empty so no parent made
+                         context=context)
   obj$class <- c("Context", obj$class)
   obj$is_realized <- function(.) TRUE
   obj$visible <- function(., value) {
@@ -292,47 +325,73 @@ aContext <- function(...,
 }
 
 ## Basic container uses a "1" column table layout
-#' A container to give a different context than the default for a set of items
-#'
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' A container to give a different context than the default for a set of items
+##'
+##' @param context ItemGroup or item to get context from. Typically just NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @seealso \code{\link{Container}}
+##' @export
+##' @examples
+##' i <- aDialog(items=list(x=numericItem(1), y=stringItem("a")))
+##' lay <- aContainer("x","y")
+##' \dontrun{i$make_gui(gui_layout=lay)}
+##' ## how to do enabled when
+##' lay <- aContainer("x",
+##'          aContainer("y", enabled_when=function(.) .$get_x() > 1))
+##' j <- i$instance()
+##' \dontrun{j$make_gui(gui_layout=lay)}
+##' ## visible can be used to hide values if not needed
+##' i <- aDialog(items=list(x=numericItem(1), y=stringItem("a")))
+##' lay <- aContainer("x","y")
+##' \dontrun{i$make_gui(gui_layout=lay)}
+##' ## how to do enabled when
+##' lay <- aContainer("x",
+##'          aContainer("y", visible_when=function(.) .$get_x() > 1))
+##' k <- i$instance()
+##' \dontrun{k$make_gui(gui_layout=lay)}
+##' 
 
-#' @examples
-#' \dontrun{
-#' i <- anItemGroup(x=numericItem(1), y=stringItem("a"))
-#' lay <- aContainer("x","y")
-#' makeGUI(i, gui_layout=lay)
-#' }
-
-aContainer <- function(..., context=NULL, enabled_when, visible_when) {
+aContainer <- function(..., context=NULL, attr=list(), enabled_when, visible_when) {
   obj <- Container$proto(children=list(...),
-                  context=context)
+                         attr=list(),
+                         context=context)
   if(!missing(enabled_when)) obj$enabled_when <- enabled_when
   if(!missing(visible_when)) obj$visible_when <- visible_when
   obj
 }
 
 ## Same as aContainer, only one can specify the number of columns
-#' A container for tabular layout
-#'
-#' @param no_cols Number of columns. Fills in row by row.
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' A container for tabular layout
+##'
+##' The basic container has one column for the item's labels and one
+##' column for the item's editors.
+##' @param no_cols Number of columns. Fills in row by row.
+##' @param context ItemGroup or item to get context from. Typically just NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @seealso \code{\link{aContainer}} constructor, \code{\link{Container}} base trait
+##' @export
+##' @examples
+##' ## simple example
+##' i <- aDialog(items=list(x=numericItem(1), y=stringItem("a")))
+##' lay <- aTableLayout("x","y", no_cols=2)
+##' \dontrun{i$make_gui(gui_layout=lay)}
+
 
 aTableLayout <- function(..., no_cols=1,# no_cols is really 2 * no_cols, we don't count labels here
-                         context=NULL, enabled_when, visible_when) { 
+                         context=NULL, attr=list(), enabled_when, visible_when) { 
   obj <- Container$proto(class=c("TableContainer", Container$class),
-                  children=list(...),
-                  no_cols=no_cols,
-                  context=context)
+                         children=list(...),
+                         no_cols=no_cols,
+                         context=context,
+                         attr=attr)
   if(!missing(enabled_when)) obj$enabled_when <- enabled_when
   if(!missing(visible_when)) obj$visible_when <- visible_when
   obj
@@ -340,48 +399,66 @@ aTableLayout <- function(..., no_cols=1,# no_cols is really 2 * no_cols, we don'
 }
 
 ## a box container
-#' A box container. Packs in items left to right or top to bottom
-#'
-#' @param horizontal If \code{TRUE} left to right, if \code{FALSE} top to bottom
-#' @param spacing Space in pixels between items
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' A box container. Packs in items left to right or top to bottom
+##'
+##' @param horizontal If \code{TRUE} left to right, if \code{FALSE} top to bottom
+##' @param spacing Space in pixels between items
+##' @param context ItemGroup or item to get context from. Typically just NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @seealso \code{\link{Container}}
+##' @export
+##' @examples
+##' i <- aDialog(items=list(xlong=numericItem(1), y=stringItem("a")))
+##' lay <- aGroup("xlongname","y", horizontal=FALSE)  # not in nice layout
+##' \dontrun{i$make_gui(gui_layout=lay)}
+##' 
 
 aGroup <- function(..., horizontal=TRUE, spacing=10,
-                   context=NULL, enabled_when, visible_when) {
+                   context=NULL, attr=list(), enabled_when, visible_when) {
   obj <- Container$proto(children=list(...),
                   type="ggroup",
-                  attr=list(horizontal=horizontal, spacing=spacing),
-                  context=context
+                  context=context, attr=attr
                   )
+  ## issue with lazy eval if done in constructor  
+  obj$attr <- merge(list(horizontal=horizontal, spacing=spacing), attr, overwrite=FALSE) 
+
   if(!missing(enabled_when)) obj$enabled_when <- enabled_when
   if(!missing(visible_when)) obj$visible_when <- visible_when
   obj
 }
 
-#' Box container with label and visual separator to indicate grouping
-#'
-#' @param label label for frame
-#' @param horizontal If \code{TRUE} left to right, if \code{FALSE} top to bottom
-#' @param spacing Space in pixels between items
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' Box container with label and visual separator to indicate grouping
+##'
+##' @param label label for frame
+##' @param horizontal If \code{TRUE} left to right, if \code{FALSE} top to bottom
+##' @param spacing Space in pixels between items
+##' @param context ItemGroup or item to get context from. Typically just NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @seealso \code{\link{Container}}
+##' @export
+##' @examples
+##'
+##' i <- aDialog(items=list(x=numericItem(1), y=stringItem("a")))
+##' lay <- aFrame(label="label frame",
+##'               aContainer("x","y"))
+##' \dontrun{i$make_gui(gui_layout=lay)}
+##'
 
 aFrame <- function(..., label="frame label", horizontal=FALSE, spacing=10,
-                   context=NULL, enabled_when, visible_when) {
+                   context=NULL, attr=list(), enabled_when, visible_when) {
   obj <- Container$proto(class=c("Frame",Container$class),
                   children=list(...),
                   type="gframe",
-                  attr=list(text=label, horizontal=horizontal, spacing=spacing),
                   context=context)
+  obj$attr <- merge(list(text=label, horizontal=horizontal, spacing=spacing), attr)
 
   if(!missing(enabled_when)) obj$enabled_when <- enabled_when
   if(!missing(visible_when)) obj$visible_when <- visible_when
@@ -391,79 +468,110 @@ aFrame <- function(..., label="frame label", horizontal=FALSE, spacing=10,
 
 ## XXX Need to work out how update_ui is done, put in model
 ## An expanding group with a trigger to show/hide its children
-#' Expanding group. Has trigger to show/hide its children
-#'
-#' @param label label for trigger
-#' @param horizontal If \code{TRUE} left to right, if \code{FALSE} top to bottom
-#' @param expanded Initial state of children. Set to \code{TRUE} to show
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' Expanding group. Has trigger to show/hide its children
+##'
+##' @param label label for trigger
+##' @param horizontal If \code{TRUE} left to right, if \code{FALSE} top to bottom
+##' @param expanded Initial state of children. Set to \code{TRUE} to show
+##' @param context ItemGroup or item to get context from. Typically just NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @seealso \code{\link{Container}}
+##' @export
+##' @examples
+##' 
+##' i <- aDialog(items=list(x=numericItem(1), y=stringItem("a")))
+##' lay <- anExpandGroup(label="label frame",
+##'                     aContainer("x","y"))
+##' \dontrun{i$make_gui(gui_layout=lay)}
+##' 
 
 anExpandGroup <- function(..., label="", horizontal=FALSE, expanded=TRUE,
-                          context=NULL, enabled_when, visible_when) {
+                          context=NULL, attr=list(), enabled_when, visible_when) {
   ## set expanded property to toggle
   obj <- Container$proto(children=list(...),
                   type="gexpandgroup",
-                  attr=list(text=label, horizontal=horizontal),
                   context=context,
                   expanded=expanded,
                   update_ui=function(.) {
                     visible(.$container) <- .$expanded
                   })
+  obj$attr <- merge(list(text=label, horizontal=horizontal), attr)
 
   if(!missing(enabled_when)) obj$enabled_when <- enabled_when
   if(!missing(visible_when)) obj$visible_when <- visible_when
   obj
 }                    
 
-#' A two panel paned group container.
-#'
-#' @param horizontal If \code{TRUE} left to right, if \code{FALSE} top to bottom
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' A two panel paned group container.
+##'
+##' @param horizontal If \code{TRUE} left to right, if \code{FALSE} top to bottom
+##' @param context ItemGroup or item to get context from. Typically just NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @seealso \code{\link{Container}}
+##' @export
+##' @examples
+##' \dontrun{
+##' i <- aDialog(items=list(x=numericItem(1), y=stringItem("a")))
+##' lay <- aPanedGroup("x","y") ## just two children,
+##' i$make_gui(gui_layout=lay)
+##' ## can put other children into a container to make just two children for aPanedGroup instance
+##' j <- aDialog(items=list(x=numericItem(1), y=stringItem("a"), z=trueFalseItem(TRUE, label="check me")))
+##' lay <- aPanedGroup("x", aContainer("y", "z"))
+##' j$make_gui(gui_layout=lay)
+##' }
 
 aPanedGroup <- function(..., horizontal=TRUE,
-                        context=NULL, enabled_when, visible_when) {
+                        context=NULL, attr=list(), enabled_when, visible_when) {
   obj <- Container$proto(children=list(...),
                   type="gpanedgroup",
-                  attr=list(horizontal=horizontal),
                   context=context,
                   validate_ui=function(.) {
                     if(length(.$children) !=2)
                       warning("aPanedGroup should have two children only")
                   }
                   )
+  obj$attr <- merge(list(horizontal=horizontal), attr)
   
   if(!missing(enabled_when)) obj$enabled_when <- enabled_when
   if(!missing(visible_when)) obj$visible_when <- visible_when
   obj
 }
 
-#' A notebook container.
-#'
-#' Pages of notebook are aNotebookPage container, which in turn can hold other items
-#' @param close_buttons Logical indicating if close buttons should be added (RGtk2 only)
-#' @param initial_page Which page to open on
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' A notebook container.
+##'
+##' Pages of notebook are made with aNotebookPage container, which in turn can hold other items.
+##' @param close_buttons Logical indicating if close buttons should be added (RGtk2 only)
+##' @param initial_page Which page to open on
+##' @param context ItemGroup or item to get context from. Typically left as NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @seealso \code{\link{Container}}
+##' @export
+##' @examples
+##' \dontrun{
+##' i <- aDialog(items=list(x=numericItem(1), y=stringItem("a")))
+##' lay <- aNotebook(
+##'           aNotebookPage(label="page 1", "x"),
+##'           aNotebookPage(label="page 2", "y")
+##'         )
+##' i$make_gui(gui_layout=lay)
+##' }
 
 aNotebook <- function(..., close_buttons=FALSE, initial_page=1,
-                      context=NULL, enabled_when, visible_when) {
+                      context=NULL, attr=list(expand=TRUE),  enabled_when, visible_when) {
   obj <- Container$proto(children=list(...),
                          type="gnotebook",
-                         attr=list(closebuttons=close_buttons),
                          initial_page=initial_page,
                          context=context,
                          validate_ui = function(.) {
@@ -479,26 +587,31 @@ aNotebook <- function(..., close_buttons=FALSE, initial_page=1,
                          },
                          init_ui = function(.) svalue(.$container) <- initial_page
                          )
-  
+  obj$attr <- merge(list(closebuttons=close_buttons), attr)
+
   if(!missing(enabled_when)) obj$enabled_when <- enabled_when
   if(!missing(visible_when)) obj$visible_when <- visible_when
 
   return(obj)
 }
 
-#' A page in a notebook
-#'
-#' @param label Tab label
-#' @param context ItemGroup or item to get context from
-#' @param enabled_when Method to determine when items in container should be enabled
-#' @param visible_when Method to determine when items in container should be visible
-#' @param ... children items specified by character strings
-#' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
-#' @export
+##' A page in a notebook
+##'
+##' Container to hold a page within a notebook container
+##' @param label Tab label for notebook page
+##' @param context ItemGroup or item to get context from. Typically just NULL.
+##' @param attr gWidget values passed to constructor
+##' @param enabled_when Method to determine when items in container should be enabled
+##' @param visible_when Method to determine when items in container should be visible
+##' @param ... children items specified by character strings
+##' @return Returns a \code{proto} object. Call \code{obj$show_help()} to view its methods and properties.
+##' @rdname aNotebook
+##' @export
 
 aNotebookPage <- function(..., label,
-                          context=NULL, enabled_when, visible_when) {
-  obj <- aContainer(..., context)
+                          context=NULL, attr=list(),
+                          enabled_when, visible_when) {
+  obj <- aContainer(..., context=context, attr=attr)
   obj$add_class("NotebookPage")
   
   obj$label <- label
